/***********************************************************
 * $Id: fancysaneexception.h 1573 2015-03-08 21:14:01Z fancylibs $
 *
 * FancyLibs sane wrapper
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSANEEXCEPTION_H_
#define FANCYSANEEXCEPTION_H_

#include <fancyexception.h>
#include <fancysanelog.h>

namespace Fancy {
namespace Sane {

class FANCYSANE_API SaneException : public Fancy::Exception
{
  int status;
#ifdef WIN32
  void *errMsg;
  void make_err_msg();
#endif

public:
    /**
     * Construct a SaneException for the given SANE
     * error code under Linux.
     *
     * Under Win32, the given status code is a HRESULT
     * value of a COM/OLE call.
     */
    SaneException (int facility,
            int priority,
	        int _status,
            const char *fmt, ...) throw()
#ifdef __GNUG__
            __attribute__((format(printf,5,6)))
#endif
;

    SaneException (int facility,
            int priority,
            const char *fmt, ...) throw()
#ifdef __GNUG__
            __attribute__((format(printf,4,5)))
#endif
;

    virtual ~SaneException() throw();

    /**
     * Returns the server side exception code, which start with 0,
     * if SANE_Status is OK returns true.
     *
     * SANE_STATUS_GOOD = 0,      everything A-OK
     * SANE_STATUS_UNSUPPORTED,   operation is not supported
     * SANE_STATUS_CANCELLED,     operation was cancelled
     * SANE_STATUS_DEVICE_BUSY,   device is busy; try again later
     * SANE_STATUS_INVAL,         data is invalid (includes no dev at open)
     * SANE_STATUS_EOF,           no more data available (end-of-file)
     * SANE_STATUS_JAMMED,        document feeder jammed
     * SANE_STATUS_NO_DOCS,       document feeder out of documents
     * SANE_STATUS_COVER_OPEN,    scanner cover is open
     * SANE_STATUS_IO_ERROR,      error during device I/O
     * SANE_STATUS_NO_MEM,        out of memory
     * SANE_STATUS_ACCESS_DENIED  access to resource has been denied
     *
     * Under Windows, this is HRESULT error code from the last remote call.
     */
	int getStatus() const throw();

	const char *getStatusMessage() const throw();

	void appendSaneMessageV(const char *msg, va_list ap) throw ();

#ifdef __GNUG__
	__attribute__((format(printf,2,3)))
#endif
    void appendSaneMessage(const char *msg, ...) throw ();
};

}}// end namespaces


#endif
